@@ -5,9 +5,8 @@ use flate2::read::GzDecoder;
 use hyper::body::HttpBody;
 use io::Read;
 
-#[allow(unused_imports)]
-use crate::_prelude::*;
 use crate::{
+	_prelude::*,
 	hyper_utils,
 	resolver::{Adaptor as ResolverAdaptor, Resolver},
 	types::*,
@@ -359,7 +358,11 @@ impl<JS: JobStateValues, TS: TaskStateValues, P: ParsedDocument> TaskProcessor<J
 		TracingTask::new(span!(n=n, url=%self.job.url), async move {
 			let (client, mut stats) = self.client_factory.make();
 
-			let mut timeout = self.job.ctx.timeout_remaining(*self.job.settings.job_hard_timeout);
+			let jitter_ms = {
+				let mut rng = thread_rng();
+				Duration::from_millis(rng.gen_range(0..self.job.settings.job_hard_timeout_jitter.as_millis()) as u64)
+			};
+			let mut timeout = self.job.ctx.timeout_remaining(*self.job.settings.job_hard_timeout + jitter_ms);
 
 			while let Ok(task) = self.tasks_rx.recv_async().await {
 				if self.tasks_rx.is_disconnected() {
